---
title: Stefan Dresselhaus
...

# Work-Experience

- **Oct. 2018 to Aug. 2021**:
    - ML-Specialist at [Jobware](https://jobware.de) (Paderborn; german Job-Advertising-Platform)
        - Extraction/Classification of sentences from JobAds (Requirements, Benefits, Tasks, ...)
        - Extraction of Information from JobAds (Location of company, Location of workplay, contact-details, application-procedure, etc.) including geocoding of those information (backed by OpenStreetMap)
        - Embedding of JobAds into a meaningful space (i.e. "get me similar ads. btw. i dislike ad a, b, c").
        - Analyse & predict search-queries of users on the webpage and offer likely but distinct queries (i.e. similar when typo or complete different words (synonyms, hyponyms, etc.))
    - Technologies used:
        - Haskell (currently GHC 8.6, soon GHC 8.8)
            - stack + stackage-lts
            - fixplate (recursion-schemes-implementation)
            - many usual technologies like lens, http-simple, mtl, ..
            - golden-testing via tasty
            - several inhouse-developments:
                - templating based on text-replacement via generics (fieldname in Template-Type == variable replaced in template)
                - activeMQ/Kibana-bridge for logging via hs-stomp
                - generic internal logging-framework
        - Python
            - tensorflow
            - pytorch
            - sklearn
            - nltk

- **2013-2018**:
    - several jobs at my University including
        - Worked 6 Months in the Workgroup "Theoretical Computer Science" on migrating algorithms to **CUDA**
        - Tutor "Introduction to Machine Learning"
            - Was awarded **Tutoring-Award** of the Faculty of Technology for excellent tutoring
        - Lecture "Intermediate Functional Programming in Haskell"
            - Originally developed as student-project in cooperation with Jonas Betzendahl
            - First held in Summer 2015
            - Due to high demand held again in Summer 2016 and 2017
            - Was awarded **Lecturer-Award** "silver Chalk" in 2016
                - First time that this award was given to students
                - Many lecturers at our faculty never get any teaching-award until retirement
        - Development of Pandoc-Filters for effective **generation of lecture-slides** for Mario Botsch (Leader Workgroup Computer Graphics) using Pandoc & reveal.js
            - Framework: [https://github.com/mbotsch/revealSlides](https://github.com/mbotsch/revealSlides)
            - Example: [https://github.com/mbotsch/eLearning](https://github.com/mbotsch/eLearning)
            - Pandoc-Filters: [https://github.com/mbotsch/pandoc-slide-filter](https://github.com/mbotsch/pandoc-slide-filter)

<img align="right" style='border:1px solid #000000; float:right; margin-left:20px' height='300px' src="/About/DresselhausStefan_klein2.jpg"/>

# Education

- **Bachelor** "Kognitive Informatik" (Cognitive Informatics) in Bielefeld 2010-2014
- **Master** "Naturwissenschaftliche Informatik" (Informatics in the natural sciences) 2014-2018

## Extraordinary grades (Excerpt of my Transcript)
    Scale of grades in Germany is 1.0 to 4.0 with 1.0 being best, 4.0 being passing grade, 5.0 being failed grade

- **1.0 in Modern Data Analysis**
    - Master course on data-analysis (time-series, core-vector-machines, gaussian processes, ...)
- **1.0 in Computergraphics**
    - Raytracing, Modern OpenGL
- **1.3 in Computer-Animation**
    - Dual-Quarternion-Skinning, Character-Animation, FACS-Poses, etc.
- **1.3 in GPU-Computing (CUDA)**
    - originally a 1.7 by timing (task was de-mosaicing on images, grade was measured in ms, whereby 400ms equated to 4.0 and 100ms equated to 1.0), but because my deep knowledge was visible in the code i was given a 1.3
- **1.0 in Parallel Algorithms and Data-Structures**
- **Ethical Hacking**
    - Reverse Engineering with IDApro

# Haskell-Enthusiast

- Learning/Writing Haskell since ~2014
- Created and held advanced Haskell-Lecture at my University

## github

- [My Profile](https://github.com/Drezil/)
- [Haskell-Lecture](https://github.com/FFPiHaskell/)
- [Co-Founder of DataHaskell](https://github.com/DataHaskell)

## Highlights on github

- **Author** of Eve-Online-Interface in [yesod-auth-oauth2](https://github.com/thoughtbot/yesod-auth-oauth2/pull/33)
- **Author** of "New Eden Accounting Tool" ([neat](https://github.com/Drezil/neat)), which is basically a ledger for Trading in the game Eve-Online
- Driver behind getting [https://github.com/jgm/pandoc/issues/168]() implemented and merged, because we needed it for our slide-filters (see Work->Development of Filters)
- **Author** of [img2ascii](https://github.com/Drezil/img2ascii) - Small cli-tool for converting images into terminal-codes & ascii using JuicyPixels, because i always forget what is on the images over an ssh-connection -.-
- **Implemented Array-Fusion and Recycling** for [subhask](https://github.com/mikeizbicki/subhask/pull/57) as layed out in [Recycle your Arrays](https://doi.org/10.1007/978-3-540-92995-6_15) by Roman Leshchinskiy
- [**Raytracer** in Haskell for my Computergraphics-Course](https://github.com/Drezil/htrace)
- **implementation of [Densely Connected Bi-Clusters](https://github.com/Drezil/hgraph)-Algorithm** in Haskell ([Paper](https://www.researchgate.net/profile/Recep_Colak/publication/267918524_DENSELY-CONNECTED_BI-CLUSTERING/links/560f1aff08ae483375178a03.pdf))
- several other dead projects :D


# Studium generale / University-Life

(What I did at university besides studying ;) )

## Committees / Student Body

- Student Member of Studienbeirat Informatik (Study-Profile Commission)
- Student Member of Tutorenauswahlkommission (Tutor-Selection Committee)
    - Leader Tutorenevaluation (Evaluation of Tutors)
- Student Member of NWI-Master-Auswahlausschuss (Master-Application Committee for my course of study)
- Student Member of NWI-Master-Prüfungsausschuss (Committee for Exam-disputes of my Master course)
- Member of the Admin-Team for the student-body pcs

## ekvv-Links (entries in the electronic course-catalog)


### Summer 15

- [Fortgeschrittene funktionale Programmierung in Haskell](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=54004629) (Haskell-Lecture)

    - [Lecture on YouTube](https://www.youtube.com/playlist?list=PLMqFm6rr-xOWhXGroUXzWx00FeaBNfbsa)

### Summer 16

- [Fortgeschrittene funktionale Programmierung in Haskell](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=71172682) (Haskell-Lecture)

    - [Lecture on YouTube](https://www.youtube.com/playlist?list=PLMqFm6rr-xOUEf2YjSxRn8BIhrdRIhZw6) (differs from link above)
    - This was the **"silver chalk"-lecture**

### Winter 16/17

- [Richtig Starten](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=84763664) (Start Right!)
- [Tutor Introduction to Machine Learning](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=79599350) (Tutor in this Lecture)
	- Was awarded **Tutoring-Award** of the faculty
- Remade and updated slides for [Computergraphics-Lecture](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=79016005)
	- Lecture was **awarded "silver chalk"** among others things because of the updated slides.

### Summer 17
- [Fortgeschrittene funktionale Programmierung in Haskell](https://ekvv.uni-bielefeld.de/kvv_publ/publ/vd?id=94694136) (Haskell-Lecture)
    - Same as Summer 16
    - Totally **reworked Exercises** accompanying the lecture

