---
categories: Mathematik
toc: yes
title: Logik
...

Logik ist das ziehen von Schlüssen, die innerhalb der Logik widerspruchsfrei und kohärent sind.

# minimale Logik

Für eine minimale Logik braucht man folgende Annahmen:

Im folgenden ist jedes Merkmal als Dreiklang definiert, bestehend aus einer Formaldefinition (F - WAS ist das), einer Einführung (I - WIE kann ich das "erzeugen") und einer Elimination (E - WIE bekomme ich das wieder weg).

Formal kurz:

- prop ist eine Eigenschaft, die zu true oder false evaluieren kann. Wir berüchsichtigen nur ein weiteres schließen, wenn prop entweder wahr oder unevaluiert ist.
- A, B, C sind Aussagen
- $\Gamma$ sind Aussagensammlungen (eine beliebige Anzahl an Aussagen in beliebiger Reihenfolge)

Alle weiteren Operationen werden eingeführt.

## Wahrheit T

Eine Wahrheit T, auf die jede Aussage die logisch ist reduzierbar sein muss:

$\frac{}{T prop} (T-F)$

"prop" ist einfach eine Aussage.

$\frac{}{T true} (T-I)$

Es gibt keine "ultimative" Wahrheit, aus der ich trivial "wahr" ableiten kann. Normalerweise bekannt als "Axiome". Später einfach Grundannahmen.

$- (T-E)$

Es gibt keine Elimination. Etwas, was inherent Wahr ist, kann ich nicht im Nachhinein ändern (ohne zuvor einen Fehler gemacht zu haben).

## Verknüpfung $\wedge$

Eine Verknüpfung $\wedge$, die zutrifft, wenn 2 Aussagen A, B wahr sind:

$\frac{A prop; B prop}{A \wedge B prop} (\wedge F)$

A & B sind Aussagen, Ergebnis ist eine Aussage über beides

$\frac{A true; B true}{A \wedge B true} (\wedge I)$

Wenn A wahr & B wahr, dann ist beides zusammen auch wahr.

$\frac{A \wedge B true}{A true} (\wedge E_1)\quad \frac{A \wedge B true}{B true} (\wedge E_2)$

Wenn A & B wahr sind, dann auch die Teile

## Implikation $\supset$

Eine Implikation (B ist in A enthalten):

$\frac{A prop; B prop}{A \supset B prop} (\supset F)$

A & B sind Aussagen, Ergebnis ist eine Aussage über beides

$\frac{A true \vdash B true}{A \supset B true} (\supset I)$

Wenn A B impliziert, dann ist A eine größere Aussage als B.

$\frac{A \supset B true; A true}{B true} (\supset E)$

Wenn A B umfasst und A wahr ist, dann muss B wahr sein.

## Entailment $\vdash$

Was soll nun dieser $\vdash$-Operator sein? Nun, dass ist "folgerbarkeit". Folgende Bedingungen **müssen** erfüllt sein, damit das (für uns) sinnvoll ist:

1.  $A true \vdash A true$ (Reflexiv, Wenn A wahr ist, dann folgt daraus, dass A wahr ist!)
2.  $\frac{\Gamma_1 \vdash A true; \Gamma_2, A true \vdash B true}{\Gamma_1, \Gamma_2 \vdash B true}$ (Transitiv, Wenn A unter Bedingung X wahr ist, und B unter Bedingung AY, dann ist B auch unter XY wahr. Wenn 1 + 1 = 2 und 3+2 = 5, dann auch 3 + (1+1) = 5)

und folgendes **sollte** erfüllt sein, außer wir bewegen uns auf ganz krudem Terrain:

3.  $\frac{\Gamma \vdash A true}{\Gamma, B true \vdash A true}$ (Weakening. Wenn A schon bewiesen ist, dann wird es nicht falsch durch hinzufügen einer wahren Vorraussetzung (die für den Beweis nicht nötig wäre)).
4.  $\frac{\Gamma, A true, A true \vdash B true}{\Gamma, A true \vdash B true}$ (Contraction. Wenn ich A 2x verwende, dann kann ich es auch nur 1x oben drüber schreiben).
5.  $\frac{\Gamma \vdash A true}{\Pi(\Gamma) A true}$ (Permutation. Egal in welcher Permutation $\pi$ ich die Argumente aufführe, der Schluss ist gleich).

## Unwahrheit $\perp$

Eine Aussage, die **nie** Eintreten darf. Wenn wird dieses Schlussfolgern *können*, dann haben wir einen Fehler

$\frac{}{\perp prop} (\perp F)$

bottom ist eine Aussage

$-(\perp I)$

bottom kann nicht Abgeleitet werden

$\frac{\perp true}{A true} (\perp E)$

Wenn wir bottom Ableiten können, gilt alles (1=0, Gott existiert und existiert nicht zugleich, ...).

## Disjunktion $\vee$

$\frac{A prop, B prop}{A \vee B prop} (\vee F)$

A & B sind Aussagen, A oder B ist auch eine Aussage

$\frac{A true}{A \vee B true} (\vee I_1)\quad \frac{B true}{A \vee B true} (\vee I_2)$

Wenn eins von beidem Wahr ist, dann ist A oder B wahr. Sprich, ich kann einfach irgendeine Aussage (egal ob wahr oder falsch) zu einer Aussage hinzufügen und die Disjunktion bleibt gleich.

$\frac{A \vee B true, A true \vdash C true, B true \vdash C true}{C true} (\vee E)$

Wenn C aus A folgt UND C aus B folgt UND entweder A oder B wahr sind, ist C wahr. OB jetzt entweder A oder B wahr sind interessiert für den Schluss nicht. Es reicht, dass Eines von beiden wahr ist.

# Folgerungen

1.  Eine Aussage A ist immer $\perp \le A \le T$. Sie ist also wahr oder falsch (=trivial) oder je nach Bedingung anders.
2.  Es gibt eine "Mächtigkeitsreihenfolge" bei Aussagen. Die Aussagen, die mit den wenigsten Vorraussetzungen auskommen sind mächtiger und enthalten mehr Aussagen, als die speziellen.
3.  Zu einer Aussage A gibt es eine Gegenaussage $\neg A := A \supset \perp$. Diese ist genau dann falsch, wenn A wahr ist. Formal: $\frac{A \wedge C \le \perp }{C \le \neg A}$. Wenn A widerlegt werden kann, ist $\neg A$ die mächtigste Widerlegung.
4.  Zu einer Aussage A gibt es ein Inverses $\bar{A}$, sodass $A \vee \bar{A} \simeq T$, also $T \le A \vee \bar{A}$. $\bar{A}$ ist somit die größte Aussage, die dazu führt, dass sie mit A ver-odert wahr wird.
5.  **Bemerkenswert ist:** Im Allgemeinen gilt *nicht*: $\neg A = \bar{A}$.

Zu 5. noch eine Bemerkung: Dies ist trivial klar, wenn man sich bewusst macht, dass nicht jedes Problem gelöst ist. Dies sind Dinge, über die die Logik alleine keine Aussage treffen KANN. Allerdings müssen wir Berücksichtigen, dass es solche Annahmen GIBT.

Ein Beispiel für eine Logik, die Annahme 5 macht: Bool'sche Algebra

Ein Beispiel für eine Logik, die Annahme 5 *nicht* macht: Heyting Algebra, u.U. auch Lindenbaum Algebra (nicht geprüft)

Damit ist letztere qua definitionem mächtiger als Erstgenannte, da diese in letztgenannter enthalten ist.